﻿using Hotline.FlowEngine.Workflows;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using MapsterMapper;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.KnowledgeBase
{
    public class KnowledgeDomainService : IKnowledgeDomainService, IScopeDependency
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly IMapper _mapper;
        private readonly IKnowledgePVRepository _knowledgePVRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="knowledgePVRepository"></param>
        public KnowledgeDomainService(IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository, IMapper mapper, IKnowledgePVRepository knowledgePVRepository)
        {
            _knowledgeRepository = knowledgeRepository;
            _mapper = mapper;
            _knowledgeTempRepository = knowledgeTempRepository;
            _knowledgePVRepository = knowledgePVRepository;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task<Knowledge> KnowledgeInfo(string Id, CancellationToken cancellationToken)
        {
            var know = await _knowledgeRepository.GetAsync(Id, cancellationToken);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            return know;
        }

        /// <summary>
        /// 浏览记录写入
        /// </summary>
        /// <param name="know"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task KnowledgePvIncreaseAsync(Knowledge know, CancellationToken cancellationToken)
        {
            //写入浏览记录
            KnowledgePv knowledgePv = new()
            {
                CreationTime = DateTime.Now,
                KnowledgeId = know.Id
            };
            //浏览记录写入
            await _knowledgePVRepository.AddAsync(knowledgePv, cancellationToken);

            //修改浏览量
            know.PageView++;
            //修改点击量
            await _knowledgeRepository.UpdateAsync(know, cancellationToken);
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<string> KnowledgeAdd(AddKnowledgeDto dto, CancellationToken cancellationToken)
        {
            var kn = _mapper.Map<Knowledge>(dto);
            //知识草稿ID如果不为空，则表示是草稿箱的数据
            if (!string.IsNullOrEmpty(dto.Code))
            {
                //移除草稿箱数据
                await _knowledgeTempRepository.RemoveAsync(p => p.Code == dto.Code, false, cancellationToken);
            }
            //如果草稿箱不存在数据，也就是Code为空，从新生成Code
            if (string.IsNullOrEmpty(kn.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.Status = EKnowledgeStatus.Auditing;

            return await _knowledgeRepository.AddAsync(kn, cancellationToken);
            //调用流程创建

            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task KnowledgeUpdate(UpdateKnowledgeDto dto, CancellationToken cancellationToken)
        {
            //知识修改逻辑
            //如果可以修改，此条数据不动，重新写入一条新数据，知识编号不变，版本号+1

            var know = await _knowledgeRepository.GetAsync(dto.Id, cancellationToken);
            if (know is null || know.IsDeleted)
                throw UserFriendlyException.SameMessage("无效知识");
            //审批中数据不能处理
            if (know.Status == EKnowledgeStatus.Auditing)
                throw UserFriendlyException.SameMessage("审批中数据不能修改");

            var knowledge = _mapper.Map<Knowledge>(dto);
            knowledge.Code = know.Code;
            knowledge.PageView = 0;
            knowledge.Status = EKnowledgeStatus.Auditing;
            knowledge.Id = "";
            knowledge.Version = know.Version + 1;
            //新增一条数据
            await _knowledgeRepository.AddAsync(knowledge, cancellationToken);

            //调用流程创建

            //如果流程创建失败需要删除此条数据
            //await _knowledgeRepository.RemoveAsync(knowledge);
        }

        /// <summary>
        /// 知识库-删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        public async Task KnowledgeDelete(string Id, CancellationToken cancellationToken)
        {
            var know = await _knowledgeRepository.GetAsync(Id, cancellationToken);
            if (know != null && know.IsDeleted == false)
            {
                await _knowledgeRepository.RemoveAsync(know, true, cancellationToken);
                //调用流程创建
            }
            else
                throw UserFriendlyException.SameMessage("知识删除失败");
        }

        /// <summary>
        /// 审批操作
        /// </summary>
        /// <param name="Ids">需要审批的数据ID</param>
        /// <param name="ApprovalStatus">审核状态：0：通过；1：不通过</param>
        /// <param name="HandleOpinions">办理意见</param>
        /// <returns></returns>
        public async Task ApprovalHandle(List<string> Ids, string ApprovalStatus, string HandleOpinions, CancellationToken cancellationToken)
        {
            //审批操作
            foreach (var item in Ids)
            {
                var know = await _knowledgeRepository.GetAsync(item, cancellationToken);
                if (know != null)
                {
                    var strWorkflowId = "";
                    //调用流程审批接口
                    //流程调用成功以后在处理以下业务
                    //需要区分新增、修改、删除业务
                    //处理数据
                    if (ApprovalStatus == "0")
                    {
                        know.Status = EKnowledgeStatus.OnShelf;
                        know.OnShelfTime = DateTime.Now;
                        know.OffShelfTime = null;
                    }
                    else
                        know.Status = EKnowledgeStatus.Failed;

                    if (!string.IsNullOrEmpty(strWorkflowId))
                    {
                        know.WorkflowId = strWorkflowId;
                        await _knowledgeRepository.UpdateAsync(know, cancellationToken);
                    }
                }
            }
        }
    }
}

